@@ -1,3 +1,4 @@
+{-# LANGUAGE ExplicitForAll    #-}
 {-# LANGUAGE FlexibleContexts  #-}
 {-# LANGUAGE OverloadedStrings #-}
 
@@ -6,17 +7,20 @@
 module Network.OAuth.OAuth2.HttpClient (
 -- * Token management
   fetchAccessToken,
+  fetchAccessToken2,
   fetchRefreshToken,
+  refreshAccessToken,
   doJSONPostRequest,
   doFlexiblePostRequest,
   doSimplePostRequest,
 -- * AUTH requests
   authGetJSON,
   authGetBS,
-  authGetBS',
+  authGetBS2,
   authPostJSON,
   authPostBS,
-  authPostBS',
+  authPostBS2,
+  authPostBS3,
   authRequest,
 -- * Utilities
   handleResponse,
@@ -27,23 +31,25 @@ module Network.OAuth.OAuth2.HttpClient (
 ) where
 
 import           Data.Aeson
-import qualified Data.ByteString.Char8         as BS
-import qualified Data.ByteString.Lazy.Char8    as BSL
-import qualified Data.HashMap.Strict           as HM (fromList)
+import qualified Data.ByteString.Char8             as BS
+import qualified Data.ByteString.Lazy.Char8        as BSL
+import qualified Data.HashMap.Strict               as HM (fromList)
 import           Data.Maybe
-import qualified Data.Text.Encoding            as T
-import           Network.HTTP.Conduit          hiding (withManager)
-import qualified Network.HTTP.Types            as HT
-import           Network.HTTP.Types.URI        (parseQuery)
+import qualified Data.Text.Encoding                as T
+import           Network.HTTP.Conduit
+import qualified Network.HTTP.Types                as HT
+import           Network.HTTP.Types.URI            (parseQuery)
 import           Network.OAuth.OAuth2.Internal
-import           URI.ByteString
 import qualified Network.OAuth.OAuth2.TokenRequest as TR
+import           URI.ByteString
 
 --------------------------------------------------
 -- * Token management
 --------------------------------------------------
 
--- | Request (via POST method) "OAuth2 Token".
+-- | Request OAuth2 Token
+--   method: POST
+--   authenticate in header
 fetchAccessToken :: Manager                                   -- ^ HTTP connection manager
                    -> OAuth2                                  -- ^ OAuth Data
                    -> ExchangeToken                           -- ^ OAuth 2 Tokens
@@ -51,15 +57,34 @@ fetchAccessToken :: Manager                                   -- ^ HTTP connecti
 fetchAccessToken manager oa code = doFlexiblePostRequest manager oa uri body
                            where (uri, body) = accessTokenUrl oa code
 
+-- | Request OAuth2 Token
+--   method: POST
+--   authenticate in both header and body
+fetchAccessToken2 :: Manager                                   -- ^ HTTP connection manager
+                   -> OAuth2                                  -- ^ OAuth Data
+                   -> ExchangeToken                           -- ^ OAuth 2 Tokens
+                   -> IO (OAuth2Result TR.Errors OAuth2Token) -- ^ Access Token
+fetchAccessToken2 mgr oa code = do
+  let (url, body1) = accessTokenUrl oa code
+  let extraBody = [ ("client_id", T.encodeUtf8 $ oauthClientId oa)
+                  , ("client_secret", T.encodeUtf8 $ oauthClientSecret oa)
+                  ]
+  doFlexiblePostRequest mgr oa url (extraBody ++ body1)
 
 -- | Request a new AccessToken with the Refresh Token.
--- TODO: seems more approporate to rename to refreshAccessToken
+refreshAccessToken :: Manager                         -- ^ HTTP connection manager.
+                     -> OAuth2                       -- ^ OAuth context
+                     -> RefreshToken                 -- ^ refresh token gained after authorization
+                     -> IO (OAuth2Result TR.Errors OAuth2Token)
+refreshAccessToken manager oa token = doFlexiblePostRequest manager oa uri body
+                              where (uri, body) = refreshAccessTokenUrl oa token
+
+{-# DEPRECATED fetchRefreshToken "Use refreshAccessToken since this method will be removed in future release" #-}
 fetchRefreshToken :: Manager                         -- ^ HTTP connection manager.
                      -> OAuth2                       -- ^ OAuth context
                      -> RefreshToken                 -- ^ refresh token gained after authorization
                      -> IO (OAuth2Result TR.Errors OAuth2Token)
-fetchRefreshToken manager oa token = doFlexiblePostRequest manager oa uri body
-                              where (uri, body) = refreshAccessTokenUrl oa token
+fetchRefreshToken = refreshAccessToken
 
 
 -- | Conduct post request and return response as JSON.
@@ -72,6 +97,7 @@ doJSONPostRequest :: FromJSON err => FromJSON a
 doJSONPostRequest manager oa uri body = fmap parseResponseJSON (doSimplePostRequest manager oa uri body)
 
 -- | Conduct post request and return response as JSON or Query String.
+{-# DEPRECATED doFlexiblePostRequest "Use doJSONPostRequest since this function would be removed in future release." #-}
 doFlexiblePostRequest :: FromJSON err => FromJSON a
                          => Manager                             -- ^ HTTP connection manager.
                          -> OAuth2                              -- ^ OAuth options
@@ -116,12 +142,13 @@ authGetBS manager token url = do
   where upReq = updateRequestHeaders (Just token) . setMethod HT.GET
 
 -- | same to 'authGetBS' but set access token to query parameter rather than header
-authGetBS' :: FromJSON err => Manager                -- ^ HTTP connection manager.
+authGetBS2 :: FromJSON err => Manager                -- ^ HTTP connection manager.
              -> AccessToken
              -> URI
              -> IO (OAuth2Result err BSL.ByteString) -- ^ Response as ByteString
-authGetBS' manager token url = do
+authGetBS2 manager token url = do
   req <- uriToRequest (url `appendAccessToken` token)
+  -- print $ queryString req
   authRequest req upReq manager
   where upReq = updateRequestHeaders Nothing . setMethod HT.GET
 
@@ -148,18 +175,30 @@ authPostBS manager token url pb = do
         upReq = upHeaders . upBody
 
 -- | Conduct POST request with access token in the request body rather header
-authPostBS' :: FromJSON err => Manager               -- ^ HTTP connection manager.
+authPostBS2 :: FromJSON err => Manager               -- ^ HTTP connection manager.
              -> AccessToken
              -> URI
              -> PostBody
              -> IO (OAuth2Result err BSL.ByteString) -- ^ Response as ByteString
-authPostBS' manager token url pb = do
+authPostBS2 manager token url pb = do
   req <- uriToRequest url
   authRequest req upReq manager
   where upBody = urlEncodedBody (pb ++ accessTokenToParam token)
         upHeaders = updateRequestHeaders Nothing . setMethod HT.POST
         upReq = upHeaders . upBody
 
+-- | Conduct POST request with access token in the header and null in body
+authPostBS3 :: FromJSON err => Manager               -- ^ HTTP connection manager.
+             -> AccessToken
+             -> URI
+             -> IO (OAuth2Result err BSL.ByteString) -- ^ Response as ByteString
+authPostBS3 manager token url = do
+  req <- uriToRequest url
+  authRequest req upReq manager
+  where upBody req = req { requestBody = "null" }
+        upHeaders = updateRequestHeaders (Just token) . setMethod HT.POST
+        upReq = upHeaders . upBody
+
 -- |Send an HTTP request including the Authorization header with the specified
 --  access token.
 --
@@ -185,9 +224,9 @@ parseResponseJSON :: FromJSON err => FromJSON a
               => OAuth2Result err BSL.ByteString
               -> OAuth2Result err a
 parseResponseJSON (Left b) = Left b
-parseResponseJSON (Right b) = case decode b of
-                            Nothing -> Left (parseOAuth2Error b)
-                            Just x -> Right x
+parseResponseJSON (Right b) = case eitherDecode b of
+                            Left e  -> Left $ mkDecodeOAuth2Error b e
+                            Right x -> Right x
 
 -- | Parses a @OAuth2Result BSL.ByteString@ that contains not JSON but a Query String
 parseResponseString :: FromJSON err => FromJSON a
@@ -204,7 +243,7 @@ parseResponseString (Right b) = case parseQuery $ BSL.toStrict b of
     paramToPair (k, mv) = (T.decodeUtf8 k, maybe Null (String . T.decodeUtf8) mv)
     errorMessage = parseOAuth2Error b
 
--- | Try 'parseResponseJSON' and 'parseResponseString'
+-- | Try 'parseResponseJSON', if failed then parses the @OAuth2Result BSL.ByteString@ that contains not JSON but a Query String.
 parseResponseFlexible :: FromJSON err => FromJSON a
                          => OAuth2Result err BSL.ByteString
                          -> OAuth2Result err a
