@@ -14,12 +14,12 @@ import           Control.Applicative
 import           Control.Arrow        (second)
 import           Control.Monad.Catch
 import           Data.Aeson
-import           Data.Aeson.Types
+import           Data.Aeson.Types (explicitParseFieldMaybe, Parser)
 import qualified Data.ByteString      as BS
 import qualified Data.ByteString.Lazy as BSL
 import           Data.Maybe
-import           Data.Monoid
-import           Data.Text            (Text, pack)
+import           Data.Semigroup       ((<>))
+import           Data.Text            (Text, pack, unpack)
 import           Data.Text.Encoding
 import           GHC.Generics
 import           Lens.Micro
@@ -60,10 +60,20 @@ data OAuth2Token = OAuth2Token {
     , idToken      :: Maybe IdToken
     } deriving (Show, Generic)
 
+parseIntFlexible :: Value -> Parser Int
+parseIntFlexible (String s) = pure . read $ unpack s
+parseIntFlexible v = parseJSON v
+
 -- | Parse JSON data into 'OAuth2Token'
 instance FromJSON OAuth2Token where
-    parseJSON = genericParseJSON defaultOptions { fieldLabelModifier = camelTo2 '_' }
+    parseJSON = withObject "OAuth2Token" $ \v -> OAuth2Token
+        <$> v .: "access_token"
+        <*> v .:? "refresh_token"
+        <*> explicitParseFieldMaybe parseIntFlexible v "expires_in"
+        <*> v .:? "token_type"
+        <*> v .:? "id_token"
 instance ToJSON OAuth2Token where
+    toJSON = genericToJSON defaultOptions { fieldLabelModifier = camelTo2 '_' }
     toEncoding = genericToEncoding defaultOptions { fieldLabelModifier = camelTo2 '_' }
 
 data OAuth2Error a =
@@ -83,6 +93,7 @@ instance FromJSON err => FromJSON (OAuth2Error err) where
   parseJSON _ = fail "Expected an object"
 
 instance ToJSON err => ToJSON (OAuth2Error err) where
+  toJSON = genericToJSON defaultOptions { constructorTagModifier = camelTo2 '_', allNullaryToStringTag = True }
   toEncoding = genericToEncoding defaultOptions { constructorTagModifier = camelTo2 '_', allNullaryToStringTag = True }
 
 parseOAuth2Error :: FromJSON err => BSL.ByteString -> OAuth2Error err
@@ -135,7 +146,7 @@ accessTokenUrl' ::  OAuth2
 accessTokenUrl' oa code gt = (uri, body)
   where uri  = oauthAccessTokenEndpoint oa
         body = catMaybes [ Just ("code", encodeUtf8 $ extoken code)
-                         , (("redirect_uri",) . serializeURIRef') <$> oauthCallback oa
+                         , ("redirect_uri",) . serializeURIRef' <$> oauthCallback oa
                          , fmap (("grant_type",) . encodeUtf8) gt
                          ]
 
